#====================================================================================
# MSIX Starter Script (Bulk Version)
# Version 1.00 (06.07.2020)
# Created by Michael Steiner
# https://www.modernx.de 
# History
# Version 1.00 - Release
#====================================================================================
<#
    .SYNOPSIS
     MSIX Starter Script (Bulk Version)
    .EXAMPLE
     .FEHLT"
    MSIX Starter Script (Bulk Version)
    .LINK
     https://www.modernx.de
#>
#====================================================================================
#region Variables

if($($Host.Name) -like "* ISE *"){
    $scriptDirectory = Split-Path -Path $psise.CurrentFile.FullPath.ToLower() -Parent
}else{
    $scriptDirectory = Split-Path -Path $MyInvocation.MyCommand.Definition -Parent
}
#==================================================================
# VARIABLEN IN DIESEM BEREICH ANPASSEN!
$CertPWclear = "modernX2020"                                      # Passwort fr den Export des Self-Signed Zertifikats
$CertCN = "MST.intra"                                             # Zertifikat CN="Organisation name" 
$CertExportPath = "$scriptDirectory\MSIXCert\MSIXCert.pfx"        # Ablage des erstellten Zertifikats fr Export / Import Funktion
$MSIXVhdxSrc="\\mst.intra\dfs\MSIXAppAttach\VLC.VHDX"             # Die entsprechende VHDX Datei welche als Container fr die MSIX Datei dient und spter am System gemounted wird
$MSIXPath = "\\mst.intra\dfs\MSIXStore\"                          # Ablage fr das MSIX Paket
$PackageName = "VLC-Player-3.0.11-x64_1.0.0.0_x64__cqfpzfs0pq22w" # Name des MSIX Pakets
$ParentFolder = "MSIX"                                            # Angabe des "root" Ordners innerhalb der VHDX fr das MSIX Paket
$MSIXJunction = "C:\temp\AppAttach\"                              # Junction Point unter welchem die VHDX angehngt wird.
#==================================================================

$CertPW = ConvertTo-SecureString -String $CertPWclear -Force -AsPlainText
if([Environment]::Is64BitOperatingSystem){
    $MSIXmgrPath = "$scriptDirectory\MSIXmgr\x64"
}else{
    $MSIXmgrPath = "$scriptDirectory\MSIXmgr\x86"
}
$Parts = $PackageName.split("_")
$AppName = $Parts[0]
$VolumeName = "MSIX-" + $Parts[0]
$PackagePath = $MSIXPath + $PackageName + ".msix"

#endregion
#====================================================================================
#region Windows Form
[System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms")
[System.Reflection.Assembly]::LoadWithPartialName("System.Drawing") 
$objForm = New-Object System.Windows.Forms.Form 
$objForm.Text = "MSIX App Attach - modernX"
$objForm.Size = New-Object System.Drawing.Size(350,320)
$objForm.MaximizeBox = $false
$objForm.FormBorderStyle = 'FixedDialog'
$objForm.StartPosition = "CenterScreen"
$objForm.KeyPreview = $True
$objIcon = New-Object System.Drawing.Icon ("$scriptDirectory\Scripts\mxLogo.ico")
$objForm.Icon = $objIcon
$objForm.Add_KeyDown({
    if($_.KeyCode -eq "Enter"){
        foreach ($objItem in $objListbox.SelectedItems){
            $x += $objItem
        }
        $objForm.Close()
    }
})
$objForm.Add_KeyDown({
    if($_.KeyCode -eq "Escape"){
        $objForm.Close()
    }
})
$OKButton = New-Object System.Windows.Forms.Button
$OKButton.Location = New-Object System.Drawing.Size(135,240)
$OKButton.Size = New-Object System.Drawing.Size(75,25)
$OKButton.Text = "OK"
$OKButton.Add_Click({
    foreach($objItem in $objListbox.SelectedItems){
        $x += $objItem
    }
    $objForm.Close()
})
$objForm.Controls.Add($OKButton)
$CancelButton = New-Object System.Windows.Forms.Button
$CancelButton.Location = New-Object System.Drawing.Size(215,240)
$CancelButton.Size = New-Object System.Drawing.Size(75,25)
$CancelButton.Text = "Cancel"
$CancelButton.Add_Click({$objForm.Close()})
$objForm.Controls.Add($CancelButton)
$objLabel = New-Object System.Windows.Forms.Label
$objLabel.Location = New-Object System.Drawing.Size(50,20) 
$objLabel.Size = New-Object System.Drawing.Size(240,20) 
$objLabel.Text = "Application: $AppName"
$objForm.Controls.Add($objLabel) 
$objLabel = New-Object System.Windows.Forms.Label
$objLabel.Location = New-Object System.Drawing.Size(50,50) 
$objLabel.Size = New-Object System.Drawing.Size(240,20) 
$objLabel.Text = "Please make a selection from the list below:"
$objForm.Controls.Add($objLabel) 
$objListbox = New-Object System.Windows.Forms.Listbox 
$objListbox.Location = New-Object System.Drawing.Size(50,90) 
$objListbox.Size = New-Object System.Drawing.Size(240,20)
$objListbox.Items.Add("00-CreateCert")
$objListbox.Items.Add("01-ExportCert")
$objListbox.Items.Add("02-ImportCert")
$objListbox.Items.Add("03-PrepareImage")
$objListbox.Items.Add("04-PreStage")
$objListbox.Items.Add("05-Staging")
$objListbox.Items.Add("06-Register")
$objListbox.Items.Add("07-UnRegister")
$objListbox.Items.Add("08-UnStaging")
$objListbox.Height = 130
$objForm.Controls.Add($objListbox) 
$objForm.Topmost = $True
$objForm.Add_Shown({$objForm.Activate()})
$objForm.ShowDialog()
$AppItem = @()
foreach($objItem in $objListbox.SelectedItems){
    $AppItem += $objItem
}
#endregion
#====================================================================================
#region Create Self-Signed Cert
if($AppItem -eq "00-CreateCert"){
    Write-Host "00-CreateCert"
    New-SelfSignedCertificate -Type Custom -Subject "CN=$CertCN" -KeyUsage DigitalSignature -KeyAlgorithm RSA -KeyLength 2048 -CertStoreLocation "Cert:\LocalMachine\My" -NotAfter (Get-Date).AddYears(1) -Verbose
}
#endregion
#====================================================================================
#region Export Self-Signed Cert

if($AppItem -eq "01-ExportCert"){
    Write-Host "01-ExportCert"
    Get-ChildItem -Path "Cert:\LocalMachine\my"| Where-Object {$_.Subject -eq "CN=$CertCN"}  | Export-PfxCertificate -FilePath $CertExportPath -Password $CertPW -Verbose
}
#endregion
#====================================================================================
#region Import Self-Signed Cert

if($AppItem -eq "02-ImportCert"){
    Write-Host "02-ImportCert"
    Get-ChildItem -Path $CertExportPath | Import-PfxCertificate -CertStoreLocation "Cert:\LocalMachine\TrustedPeople" -Password $CertPW -Exportable
}
#endregion
#====================================================================================
#region PrepareImage

if($AppItem -eq "03-PrepareImage"){
    Write-Host "03-PrepareImage"

    Write-Host "Disable Store auto update:"
    New-Item -Path "HKLM:\Software\Policies\Microsoft\WindowsStore" -ErrorAction SilentlyContinue
    New-ItemProperty -Path "HKLM:\Software\Policies\Microsoft\WindowsStore" -Name "AutoDownload" -PropertyType "DWord" -Value 0 -Force
    Get-ScheduledTask -TaskPath "\Microsoft\Windows\WindowsUpdate\" -TaskName "Automatic app update" -ErrorAction SilentlyContinue | Disable-ScheduledTask -ErrorAction SilentlyContinue
    Get-ScheduledTask -TaskPath "\Microsoft\Windows\WindowsUpdate\" -TaskName "Scheduled Start" -ErrorAction SilentlyContinue | Disable-ScheduledTask -ErrorAction SilentlyContinue

    Write-Host "Disable Content Delivery auto download apps that they want to promote to users:"
    New-ItemProperty -Path "HKCU:\SOFTWARE\Microsoft\Windows\CurrentVersion\ContentDeliveryManager" -Name "PreInstalledAppsEnabled" -PropertyType "DWord" -Value 0 -Force
    New-Item -Path "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\ContentDeliveryManager" -ErrorAction SilentlyContinue
    New-Item -Path "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\ContentDeliveryManager\Debug" -ErrorAction SilentlyContinue
    New-ItemProperty -Path "HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\ContentDeliveryManager\Debug" -Name "ContentDeliveryAllowedOverride" -PropertyType "DWord" -Value 0x2 -Force

    Write-Host "Disable Windows Update:"
    Set-Service wuauserv -StartupType Disabled
    #Stop-Service wuauserv
    
    Write-Host "Enable Hyper-V Features"
    Enable-WindowsOptionalFeature -Online -FeatureName Microsoft-Hyper-V -All
    Write-Host "Restart needed!" -BackgroundColor Green

}

#endregion
#====================================================================================
#region PreStage

if($AppItem -eq "04-PreStage"){
    Write-Host "04-PreStage"

    #Generate a MSIX App Attach Container (VHDX) for MSIX
    New-VHD -SizeBytes 2048MB -Path $MSIXVhdxSrc -Dynamic -Confirm:$false 
    $VHDObject = Mount-VHD $MSIXVhdxSrc -Passthru
    $Disk = Initialize-Disk -Passthru -Number $VHDObject.Number
    $Partition = New-Partition -AssignDriveLetter -UseMaximumSize -DiskNumber $Disk.Number
    Format-Volume -FileSystem NTFS -Confirm:$false -DriveLetter $Partition.DriveLetter -Force
    $UniqueId = (Get-Volume -DriveLetter $($Partition.DriveLetter)).UniqueId
    $Path = $Partition.DriveLetter + ":" + "\" + $ParentFolder

    #Create a folder in root drive of the mounted VHDX
    New-Item -Path $Path -ItemType Directory
    Set-Volume -DriveLetter $Partition.DriveLetter -NewFileSystemLabel $VolumeName

    #Expand MSIX
    Set-Location $MSIXmgrPath
    .\msixmgr.exe -Unpack -PackagePath $PackagePath -Destination $Path -ApplyACLs

    #Show Unique ID
    Write-Host "Unique ID: $UniqueId"

    #Detach VHDX
    Dismount-VHD $MSIXVhdxSrc -Passthru

}
#endregion
#====================================================================================
#region Stage

if($AppItem -eq "05-Staging"){
    Write-Host "05-Staging"

    #Mount VHDX
    try{
        Mount-Diskimage -ImagePath $MSIXVhdxSrc -NoDriveLetter -Access ReadOnly                 
        Write-Host ("Mounting of " + $MSIXVhdxSrc + " was completed!")
    }catch{
        Write-Host ("Mounting of " + $MSIXVhdxSrc + " has failed!")
    }

    #Create Link
    if($UniqueId){
        $msixDest = $UniqueId
    }else{
         $msixDest = (Get-Volume | Where-Object {$_.FileSystemLabel -eq "$VolumeName"}).UniqueId
    }
    if(!(Test-Path $MSIXJunction)){
        New-Item -Path "C:\" -Name "temp" -ItemType "Directory" -ErrorAction SilentlyContinue
        New-Item -Path "C:\temp" -Name "AppAttach" -ItemType "directory" -ErrorAction SilentlyContinue
        Write-Host "Junction Point created!"
    }
    $MSIXJunction = $MSIXJunction + $PackageName
    CMD /C "mklink /j $MSIXJunction $msixDest"

    #Staging
    [Windows.Management.Deployment.PackageManager,Windows.Management.Deployment,ContentType=WindowsRuntime] | Out-Null
    Add-Type -AssemblyName System.Runtime.WindowsRuntime

    $asTask = ([System.WindowsRuntimeSystemExtensions].GetMethods() | Where { $_.ToString() -eq 'System.Threading.Tasks.Task`1[TResult] AsTask[TResult,TProgress](Windows.Foundation.IAsyncOperationWithProgress`2[TResult,TProgress])'})[0]
    $asTaskAsyncOperation = $asTask.MakeGenericMethod([Windows.Management.Deployment.DeploymentResult], [Windows.Management.Deployment.DeploymentProgress])
    $packageManager = [Windows.Management.Deployment.PackageManager]::new()
    $path = $MSIXJunction + "\" + $ParentFolder + "\" + $PackageName
    $path = ([System.Uri]$path).AbsoluteUri
    $asyncOperation = $packageManager.StagePackageAsync($path, $null, "StageInPlace")                                                                                      
    $task = $asTaskAsyncOperation.Invoke($null, @($asyncOperation))
    $task
}
#endregion
#====================================================================================
#region Register

if($AppItem -eq "06-Register"){
    Write-Host "06-Register"

    $WindowsAppsPath = "C:\Program Files\WindowsApps\" + $PackageName + "\AppxManifest.xml"
    #register
    Add-AppxPackage -Path $WindowsAppsPath -DisableDevelopmentMode -Register
}
#endregion
#====================================================================================
#region DeRegister

if($AppItem -eq "07-UnRegister"){
    Write-Host "07-UnRegister"

    #UnRegister
    Remove-AppxPackage -PreserveRoamableApplicationData $PackageName 
}
#endregion
#====================================================================================
#region UnStage

if($AppItem -eq "08-UnStaging"){
    Write-Host "08-UnStaging"

    #UnRegister AllUsers
    Remove-AppxPackage -AllUsers -Package $PackageName
    Set-Location $MSIXJunction 
    Remove-Item $PackageName -Recurse -Force -Confirm:$false

    #Dismount VHDX
    Dismount-Diskimage -ImagePath $MSIXVhdxSrc
}
#endregion
#====================================================================================